<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>

    <xsl:template match="/lav:views">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:choose>
                <xsl:when test="lav:view">
                    <xsl:apply-templates/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:for-each select="document('/views/')/properties/entry">
                        <xsl:apply-templates select="document(@key)/lav:views/lav:view"/>
                    </xsl:for-each>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:for-each select="document('engine/views-libraries.properties')/properties/entry">
                <xsl:apply-templates select="document(@key)/lav:views/lav:view">
                    <xsl:with-param name="authenticators" select="text()"/>
                </xsl:apply-templates>
            </xsl:for-each>
            <xsl:apply-templates select="document('engine/post-processors.xml')/lav:processors-list"/>
            <xsl:apply-templates select="document('service/authenticators.xml')/lav:authenticators-list"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="lav:view | lav:pre-renderers">
        <xsl:param name="authenticators"/>
        <xsl:copy><xsl:copy-of select="@*"/>
            <!-- set @authenticators (if not already defined) -->
            <xsl:if test="self::lav:view and not(@authenticators) and $authenticators">
                <xsl:attribute name="authenticators"><xsl:value-of select="$authenticators"/></xsl:attribute>
            </xsl:if>
            <!-- set default view name -->
            <xsl:if test="self::lav:view and not(@name)">
                <xsl:attribute name="name">_default_</xsl:attribute>
            </xsl:if>
            <!-- convert xmlns:ns to <namespace prefix="ns"/> -->
            <!-- WARNING: running this from DOM makes order of <namespace> unpredictable -->
            <xsl:for-each select="(/* | ancestor::* | descendant-or-self::*)/namespace::*
                                        [name()!='']
                                        [.!='http://www.w3.org/XML/1998/namespace']">
                <namespace prefix="{name()}" uri="{.}"/>
            </xsl:for-each>
            <!-- recursive -->
            <xsl:apply-templates select="* | text() | comment()"/>
        </xsl:copy>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="comment()">
        <xsl:comment><xsl:value-of select="."/></xsl:comment>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy><xsl:copy-of select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>